import numpy as np
import matplotlib
matplotlib.use("TkAgg")
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg, NavigationToolbar2TkAgg
from matplotlib.figure import Figure
import matplotlib.animation as animation

from tkinter import *
from tkinter import ttk

from SerialData import *
import serial
import time
import os


class CameraGUI:
    def __init__(self):
        self.root = Tk()
        self.root.title("Thermovision camera")

        # Variables
        self.com_name = StringVar()
        self.com_name.set("com10")
        self.com_button_str = StringVar()
        self.com_button_str.set("Open")
        self.min = StringVar()
        self.min.set("0")
        self.max = StringVar()
        self.max.set("80")
        self.auto = IntVar()
        self.auto.set(True)
        self.hz = IntVar()
        self.hz.set(10)
        self.freeze = IntVar()
        self.interpolation = ("bicubic","none", "nearest", "bilinear", "spline16",
                         "spline36", "hanning", "hamming", "hermite", "kaiser",
                         "quadric", "catrom", "gaussian", "bessel", "mitchell",
                         "sinc", "lanczos")
        self.chosen_interpolation = StringVar()
        self.colormap = StringVar()
        self.colormaps = ("seismic", "hot", "magma", "bwr", "afmhot",
                          "viridis", "plasma", "Greys", "Reds", "Wistia",
                          "jet", "nipy_spectral")
        self.fps_label_val = StringVar()
        self.fps_label_val.set("0.0")

        self.f = Figure(figsize=(6, 6), dpi=100)
        self.plot = self.f.add_subplot(111)
        self.bar = None
        self.canvas = FigureCanvasTkAgg(self.f, self.root)
        self.canvas.show()
        self.canvas.get_tk_widget().grid(column=0, row=0)

        self.menu = ttk.Frame(self.root)
        self.menu.grid(column=1, row=0, sticky=(N, W, E, S))
        self.menu.columnconfigure(0, weight=3)
        self.menu.columnconfigure(1, weight=1)

        menu_row = 0
        self.com_label = ttk.Label(self.menu, text="Com: ", width=10)
        self.com_label.grid(column=0, row=menu_row, sticky=(N, W, E, S))

        self.com_name_label = ttk.Entry(self.menu, width=10,
                                        textvariable=self.com_name)
        self.com_name_label.grid(column=1, row=menu_row, sticky=(N, W, E, S))

        menu_row += 1
        self.com_button = ttk.Button(self.menu, command=self.com_button_clk,
                                     textvariable=self.com_button_str)
        self.com_button.grid(column=0, row=menu_row, columnspan=2,
                             sticky=(N, W, E, S))

        menu_row += 1
        self.pause_label_0 = ttk.Label(self.menu, text=" ")
        self.pause_label_0.grid(column=0, row=menu_row, sticky=(N, W, E, S))

        menu_row += 1
        self.auto_check = ttk.Label(self.menu, text="The temperature scale")
        self.auto_check.grid(column=0, row=menu_row, columnspan=2)

        menu_row += 1
        self.min_label = ttk.Label(self.menu, text="Min: ", width=10)
        self.min_label.grid(column=0, row=menu_row, sticky=(N, W, E, S))

        self.min_entry = ttk.Entry(self.menu, width=10,
                                   textvariable=self.min)
        self.min_entry.grid(column=1, row=menu_row, sticky=(N, W, E, S))

        menu_row += 1
        self.max_label = ttk.Label(self.menu, text="Max: ", width=10)
        self.max_label.grid(column=0, row=menu_row, sticky=(N, W, E, S))

        self.max_entry = ttk.Entry(self.menu, width=10,
                                   textvariable=self.max)
        self.max_entry.grid(column=1, row=menu_row, sticky=(N, W, E, S))

        menu_row += 1
        self.auto_check = ttk.Checkbutton(self.menu, text="Auto", onvalue=1,
                                          offvalue=0, variable=self.auto)
        self.auto_check.grid(column=0, row=menu_row, columnspan=2)

        menu_row += 1
        self.colormap_label = ttk.Label(self.menu, text="Colormap:")
        self.colormap_label.grid(column=0, row=menu_row)
        self.colormap.set(self.colormaps[0])
        self.colormap_list = OptionMenu(self.menu, self.colormap, *self.colormaps)
        self.colormap_list.grid(column=1, row=menu_row)

        menu_row += 1
        self.pause_label = ttk.Label(self.menu, text=" ")
        self.pause_label.grid(column=0, row=menu_row, sticky=(N, W, E, S))

        menu_row += 1
        self.hz_label = ttk.Label(self.menu, text="Refresh rate:")
        self.hz_label.grid(column=0, row=menu_row, sticky=(N, W, E, S))

        menu_row += 1
        self.hz_radio_1 = ttk.Radiobutton(self.menu, text="1 Hz",
                                          variable=self.hz, value=1)
        self.hz_radio_1.grid(column=0, row=menu_row)
        self.hz_radio_10 = ttk.Radiobutton(self.menu, text="10 Hz",
                                           variable=self.hz, value=10)
        self.hz_radio_10.grid(column=1, row=menu_row)

        menu_row += 1
        self.int_label = ttk.Label(self.menu, text="Interpolation:")
        self.int_label.grid(column=0, row=menu_row)
        self.chosen_interpolation.set(self.interpolation[0])
        self.int_list = OptionMenu(self.menu, self.chosen_interpolation, *self.interpolation)
        self.int_list.grid(column=1, row=menu_row)

        menu_row += 1
        self.freeze_check = ttk.Checkbutton(self.menu, text="Freeze", onvalue=1,
                                            offvalue=0, variable=self.freeze)
        self.freeze_check.grid(column=0, row=menu_row, columnspan=2)

        menu_row += 1
        self.pause_label_2 = ttk.Label(self.menu, text=" ")
        self.pause_label_2.grid(column=0, row=menu_row, sticky=(N, W, E, S))

        menu_row += 1
        self.file_label = ttk.Label(self.menu, text="File: ")
        self.file_label.grid(column=0, row=menu_row, sticky=(N, W, E, S))

        self.file_name = StringVar()
        self.file_name_label = ttk.Entry(self.menu, width=10,
                                         textvariable=self.file_name)
        self.file_name_label.grid(column=1, row=menu_row, sticky=(N, W, E, S))

        menu_row += 1
        self.save_button = ttk.Button(self.menu, text="Save",
                                      command=self.save_button_clk)
        self.save_button.grid(column=0, row=menu_row, columnspan=2,
                              sticky=(N, W, E, S))

        menu_row += 1
        photo = PhotoImage(file="board.ppm")
        self.image = ttk.Label(self.menu, image=photo)
        self.image.image = photo
        self.image.grid(column=0, row=menu_row, columnspan=2, padx=20, pady=20)

        menu_row += 1
        self.fps_label = ttk.Label(self.menu, textvariable=self.fps_label_val)
        self.fps_label.grid(column=1, row=menu_row, sticky=(N, W, E, S))

        self.data = MockData("")
        self.frame = self.data.get_frame()
        self.map = self.plot.imshow(self.frame, interpolation="bilinear", vmin=0, vmax=80)
        self.bar = self.f.colorbar(self.map)
        self.t = time.time()
        self.ani = animation.FuncAnimation(self.f, self.animate, interval=950)

    def animate(self, i):
        if not self.freeze.get():
            frame = self.data.get_frame()
            if frame is not None:
                self.frame = frame

        if self.auto.get():
            min = np.min(self.frame)
            max = np.max(self.frame)
        else:
            min = to_float(self.min.get(), 0)
            max = to_float(self.max.get(), 80)

        self.ani.event_source.interval = 950./self.hz.get()

        interpolation = self.chosen_interpolation.get()
        colormap = self.colormap.get()

        self.map.set_cmap(colormap)
        self.map.set_interpolation(interpolation)
        self.map.set_data(self.frame)
        self.map.set_clim(min, max)

        t = time.time()
        fps = 1/(t-self.t)
        self.fps_label_val.set("fps is {0:.1f}\tHz".format(fps))
        self.t = t

    def start(self):
        self.root.mainloop()

    def com_button_clk(self):
        print("Clicke {}".format(self.com_name.get()))
        print(type(self.data))
        if type(self.data) is MockData:
            print("start")
            try:
                data = SerialData(self.com_name.get())
                self.data = data
                self.com_button_str.set("Close")
            except serial.SerialException:
                print("Serial error")
        else:
            print("stop")
            self.data.close()
            self.data = MockData("mock")
            self.com_button_str.set("Open")

    def save_button_clk(self):
        dir = "img"
        try:
            if not os.path.isdir(dir):
                os.mkdir(dir)
            name = "{}/{}_{}.png".format(dir, self.file_name.get(),
                                         time.time())
            self.f.savefig(name)
        except:
            print("File not saved...")


def to_float(string, default):
    try:
        return float(string)
    except ValueError:
        return default


if __name__ == "__main__":
    cameraGUI = CameraGUI()
    cameraGUI.start()
